@@ -1,4 +1,4 @@
-{-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE CPP, ScopedTypeVariables #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Network.HTTP.Base
@@ -107,7 +107,7 @@ import Network.URI
    )
 
 import Control.Monad ( guard )
-import Control.Monad.Error ()
+import Control.Monad.Error.Class ()
 import Data.Bits     ( (.&.), (.|.), shiftL, shiftR )
 import Data.Word     ( Word8 )
 import Data.Char     ( digitToInt, intToDigit, toLower, isDigit,
@@ -209,7 +209,11 @@ uriAuthPort mbURI u =
   default_http  = 80
   default_https = 443
 
+#if MIN_VERSION_base(4,13,0)
+failHTTPS :: MonadFail m => URI -> m ()
+#else
 failHTTPS :: Monad m => URI -> m ()
+#endif
 failHTTPS uri
   | map toLower (uriScheme uri) == "https:" = fail "https not supported"
   | otherwise = return ()
@@ -713,7 +717,11 @@ urlEncodeVars [] = []
 
 -- | @getAuth req@ fishes out the authority portion of the URL in a request's @Host@
 -- header.
+#if MIN_VERSION_base(4,13,0)
+getAuth :: MonadFail m => Request ty -> m URIAuthority
+#else
 getAuth :: Monad m => Request ty -> m URIAuthority
+#endif
 getAuth r = 
    -- ToDo: verify that Network.URI functionality doesn't take care of this (now.)
   case parseURIAuthority auth of
