@@ -1,4 +1,4 @@
-{-# LANGUAGE ImplicitParams, ViewPatterns, NoMonomorphismRestriction #-}
+{-# LANGUAGE ImplicitParams, ViewPatterns, NoMonomorphismRestriction, CPP #-}
 import Control.Concurrent
 
 import Control.Applicative ((<$))
@@ -12,6 +12,7 @@ import Data.Maybe (fromJust)
 import System.IO.Error (userError)
 
 import qualified Httpd
+import qualified UnitTests
 
 import Network.Browser
 import Network.HTTP
@@ -22,6 +23,7 @@ import Network.Stream (Result)
 import Network.URI (uriPath, parseURI)
 
 import System.Environment (getArgs)
+import System.Info (os)
 import System.IO (getChar)
 
 import Test.Framework (defaultMainWithArgs, testGroup)
@@ -82,6 +84,22 @@ basicPostRequest = do
               (show (Just "text/plain", Just "4", sendBody))
               body
 
+userpwAuthFailure :: (?baduserpwUrl :: ServerAddress) => Assertion
+userpwAuthFailure = do
+  response <- simpleHTTP (getRequest (?baduserpwUrl "/auth/basic"))
+  code <- getResponseCode response
+  body <- getResponseBody response
+  assertEqual "HTTP status code" ((4, 0, 1),
+                "Just \"Basic dGVzdDp3cm9uZ3B3ZA==\"") (code, body)
+  -- in case of 401, the server returns the contents of the Authz header
+
+userpwAuthSuccess :: (?userpwUrl :: ServerAddress) => Assertion
+userpwAuthSuccess = do
+  response <- simpleHTTP (getRequest (?userpwUrl "/auth/basic"))
+  code <- getResponseCode response
+  body <- getResponseBody response
+  assertEqual "Receiving expected response" ((2, 0, 0), "Here's the secret") (code, body)
+
 basicAuthFailure :: (?testUrl :: ServerAddress) => Assertion
 basicAuthFailure = do
   response <- simpleHTTP (getRequest (?testUrl "/auth/basic"))
@@ -151,7 +169,6 @@ browserOneCookie :: (?testUrl :: ServerAddress) => Assertion
 browserOneCookie = do
   (_, response) <- browse $ do
     setOutHandler (const $ return ())
-    setMaxPoolSize (Just 0) -- TODO remove this: workaround for github issue 14
     -- This first requests returns a single Set-Cookie: hello=world
     _ <- request $ getRequest (?testUrl "/browser/one-cookie/1")
 
@@ -166,7 +183,6 @@ browserTwoCookies :: (?testUrl :: ServerAddress) => Assertion
 browserTwoCookies = do
   (_, response) <- browse $ do
     setOutHandler (const $ return ())
-    setMaxPoolSize (Just 0) -- TODO remove this: workaround for github issue 14
     -- This first request returns two cookies
     _ <- request $ getRequest (?testUrl "/browser/two-cookies/1")
 
@@ -182,7 +198,6 @@ browserFollowsRedirect :: (?testUrl :: ServerAddress) => Int -> Assertion
 browserFollowsRedirect n = do
   (_, response) <- browse $ do
     setOutHandler (const $ return ())
-    setMaxPoolSize (Just 0) -- TODO remove this: workaround for github issue 14
     request $ getRequest (?testUrl "/browser/redirect/relative/" ++ show n ++ "/basic/get")
   assertEqual "Receiving expected response from server"
               ((2, 0, 0), "It works.")
@@ -192,7 +207,6 @@ browserReturnsRedirect :: (?testUrl :: ServerAddress) => Int -> Assertion
 browserReturnsRedirect n = do
   (_, response) <- browse $ do
     setOutHandler (const $ return ())
-    setMaxPoolSize (Just 0) -- TODO remove this: workaround for github issue 14
     request $ getRequest (?testUrl "/browser/redirect/relative/" ++ show n ++ "/basic/get")
   assertEqual "Receiving expected response from server"
               ((n `div` 100, n `mod` 100 `div` 10, n `mod` 10), "")
@@ -205,7 +219,6 @@ browserBasicAuth :: (?testUrl :: ServerAddress) => Assertion
 browserBasicAuth = do
   (_, response) <- browse $ do
     setOutHandler (const $ return ())
-    setMaxPoolSize (Just 0) -- TODO remove this: workaround for github issue 14
 
     setAuthorityGen authGenBasic
 
@@ -222,7 +235,6 @@ browserDigestAuth :: (?testUrl :: ServerAddress) => Assertion
 browserDigestAuth = do
   (_, response) <- browse $ do
     setOutHandler (const $ return ())
-    setMaxPoolSize (Just 0) -- TODO remove this: workaround for github issue 14
 
     setAuthorityGen authGenDigest
 
@@ -413,7 +425,7 @@ isSubsetOf xs ys = all (`elem` ys) xs
 -- first bits of result text from haskell.org (just to give some representative text)
 haskellOrgText =
   "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\
-\<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\" dir=\"ltr\">\
+\\t<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\" dir=\"ltr\">\
 \\t<head>\
 \\t\t<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" />\
 \\t\t\t\t<meta name=\"keywords\" content=\"Haskell,Applications and libraries,Books,Foreign Function Interface,Functional programming,Hac Boston,HakkuTaikai,HaskellImplementorsWorkshop/2011,Haskell Communities and Activities Report,Haskell in education,Haskell in industry\" />"
@@ -524,6 +536,8 @@ basicTests =
     , testCase "Secure GET request" secureGetRequest
     , testCase "Basic POST request" basicPostRequest
     , testCase "Basic HEAD request" basicHeadRequest
+    , testCase "URI user:pass Auth failure" userpwAuthFailure
+    , testCase "URI user:pass Auth success" userpwAuthSuccess
     , testCase "Basic Auth failure" basicAuthFailure
     , testCase "Basic Auth success" basicAuthSuccess
     , testCase "UTF-8 urlEncode" utf8URLEncode
@@ -534,9 +548,8 @@ browserTests =
     testGroup "Browser tests"
     [ testGroup "Basic"
       [
-        -- github issue 14
-        -- testCase "Two requests" browserTwoRequests
         testCase "Network.Browser example code" browserExample
+      , testCase "Two requests" browserTwoRequests
       ]
     , testGroup "Secure"
       [
@@ -581,46 +594,59 @@ port80Tests =
     [ testCase "Alternate server" browserAlt
     , testCase "Both servers" browserBoth
     , testCase "Both servers (reversed)" browserBothReversed
-    -- github issue 14
-    -- , testCase "Two requests - alternate server" browserTwoRequestsAlt
-    -- , testCase "Two requests - both servers" browserTwoRequestsBoth
+    , testCase "Two requests - alternate server" browserTwoRequestsAlt
+    , testCase "Two requests - both servers" browserTwoRequestsBoth
     ]
 
-urlRoot :: Int -> String
-urlRoot 80 = "http://localhost"
-urlRoot n = "http://localhost:" ++ show n
+data InetFamily = IPv4 | IPv6
 
-secureRoot :: Int -> String
-secureRoot 443 = "https://localhost"
-secureRoot n = "https://localhost:" ++ show n
+familyToLocalhost :: InetFamily -> String
+familyToLocalhost IPv4 = "127.0.0.1"
+familyToLocalhost IPv6 = "[::1]"
+
+urlRoot :: InetFamily -> String -> Int -> String
+urlRoot fam userpw 80 = "http://" ++ userpw ++ familyToLocalhost fam
+urlRoot fam userpw n = "http://" ++ userpw ++ familyToLocalhost fam ++ ":" ++ show n
+
+secureRoot :: InetFamily -> String -> Int -> String
+secureRoot fam userpw 443 = "https://" ++ userpw ++ familyToLocalhost fam
+secureRoot fam userpw n = "https://" ++ userpw ++ familyToLocalhost fam ++ ":" ++ show n
 
 type ServerAddress = String -> String
 
-httpAddress, httpsAddress :: Int -> ServerAddress
-httpAddress port p = urlRoot port ++ p
-httpsAddress port p = secureRoot port ++ p
+httpAddress, httpsAddress :: InetFamily -> String -> Int -> ServerAddress
+httpAddress fam userpw port p = urlRoot fam userpw port ++ p
+httpsAddress fam userpw port p = secureRoot fam userpw port ++ p
 
 main :: IO ()
 main = do
   args <- getArgs
 
-  let servers = [("httpd-shed", Httpd.shed), ("warp", Httpd.warp)]
+  let servers =
+          [ ("httpd-shed", Httpd.shed, IPv4)
+#ifdef WARP_TESTS
+          , ("warp.v6", Httpd.warp True, IPv6)
+          , ("warp.v4", Httpd.warp False, IPv4)
+#endif
+          ]
       basePortNum, altPortNum :: Int
       basePortNum = 5812
       altPortNum = 80
       numberedServers = zip [basePortNum..] servers
 
   let setupNormalTests = do
-      flip mapM numberedServers $ \(portNum, (serverName, server)) -> do
-         let ?testUrl = httpAddress portNum
-             ?secureTestUrl = httpsAddress portNum
+      flip mapM numberedServers $ \(portNum, (serverName, server, family)) -> do
+         let ?testUrl = httpAddress family "" portNum
+             ?userpwUrl = httpAddress family "test:password@" portNum
+             ?baduserpwUrl = httpAddress family "test:wrongpwd@" portNum
+             ?secureTestUrl = httpsAddress family "" portNum
          _ <- forkIO $ server portNum processRequest
          return $ testGroup serverName [basicTests, browserTests]
 
   let setupAltTests = do
-      let (portNum, (_, server)) = head numberedServers
-      let ?testUrl = httpAddress portNum
-          ?altTestUrl = httpAddress altPortNum
+      let (portNum, (_, server,family)) = head numberedServers
+      let ?testUrl = httpAddress family "" portNum
+          ?altTestUrl = httpAddress family "" altPortNum
       _ <- forkIO $ server altPortNum altProcessRequest
       return port80Tests
 
@@ -635,8 +661,8 @@ main = do
         normalTests <- setupNormalTests
         altTests <- setupAltTests
         _ <- threadDelay 1000000 -- Give the server time to start :-(
-        defaultMainWithArgs (normalTests ++ [altTests]) args
+        defaultMainWithArgs (UnitTests.unitTests ++ normalTests ++ [altTests]) args
      args -> do -- run the test harness as normal
         normalTests <- setupNormalTests
         _ <- threadDelay 1000000 -- Give the server time to start :-(
-        defaultMainWithArgs normalTests args
+        defaultMainWithArgs (UnitTests.unitTests ++ normalTests) args
